/*-----------------------------------------------------------------------------+
| main.cpp                                                                     |
+-----------------------------------------------------------------------------*/


#define _WIN32_IE 0x400 // Pour certaines fonctions
#include <windows.h> // API Windows
#include <commctrl.h> // API contrles
#include <process.h> // Threads
#include <string> // chaines de caractres
#include <iostream> // Fichiers
#include <fstream> // Fichiers
#include <stdio.h>
#include "resource.h" // Identifiants ressource
#include "base64.h" // Bibliothque base64

using namespace std;
#define WM_CONVERT WM_USER+1
#define JOB_NONE 0
#define JOB_ENCODE 1
#define JOB_DECODE 2


/*-----------------------------------------------------------------------------+
| Varaibles globales                                                           |
+-----------------------------------------------------------------------------*/
static ifstream Input; // Fichier lecture
static ofstream Output; // Fichier criture
static unsigned long long FileSize = 0; // Taille de fichier
static int CallBackJob = JOB_NONE; // Identifiant du job d'arrire-plan
static const char cb64[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
static const char cd64[]="|$$$}rstuvwxyz{$$$$$$$>?@ABCDEFGHIJKLMNOPQRSTUVW$$$$$$XYZ[\\]^_`abcdefghijklmnopq";


/*-----------------------------------------------------------------------------+
| Dialog d'ouverture de fichier                                                |
+-----------------------------------------------------------------------------*/
string OpenFileBox(string Title, char* Filter, int Index, long Flags, HWND hDlg)
{
  OPENFILENAME ofn;
  char szFile[MAX_PATH] = {0};
  ZeroMemory(&ofn, sizeof(OPENFILENAME));
  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner = hDlg;
  ofn.lpstrFile = szFile;
  ofn.nMaxFile = MAX_PATH;
  ofn.lpstrTitle = Title.c_str();
  ofn.lpstrFilter = Filter;
  ofn.nFilterIndex = Index;
  ofn.Flags = OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST|OFN_HIDEREADONLY|Flags;
  if (GetOpenFileName(&ofn) == TRUE)
    return string(szFile);
  return "";
}


/*-----------------------------------------------------------------------------+
| Dialog d'ouverture de fichier                                                |
+-----------------------------------------------------------------------------*/
string SaveFileBox(string Title, char* Filter, int Index, long Flags, HWND hDlg)
{
  OPENFILENAME ofn;
  char szFile[MAX_PATH] = {0};
  ZeroMemory(&ofn, sizeof(OPENFILENAME));
  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner = hDlg;
  ofn.lpstrFile = szFile;
  ofn.nMaxFile = MAX_PATH;
  ofn.lpstrTitle = Title.c_str();
  ofn.lpstrFilter = Filter;
  ofn.nFilterIndex = Index;
  ofn.Flags = OFN_PATHMUSTEXIST|OFN_HIDEREADONLY|Flags;
  if (GetSaveFileName(&ofn) == TRUE)
    return string(szFile);
  return "";
}


/*-----------------------------------------------------------------------------+
| Rcupre la taille de fichier                                                |
+-----------------------------------------------------------------------------*/
unsigned long long GetFileSize(std::ifstream &Fichier)
{
  unsigned long long pos = Fichier.tellg();
  Fichier.seekg(0, std::ios_base::end);
  unsigned long long size = Fichier.tellg();
  Fichier.seekg(pos, std::ios_base::beg);
  return size;
}


/*-----------------------------------------------------------------------------+
| Finalisation de la conversion                                                |
+-----------------------------------------------------------------------------*/
void OnEndConversion(HWND hDlg)
{
  // Dsactive la conversion
  CallBackJob = JOB_NONE;
  KillTimer(hDlg, 0);
  // Modifie les textes des boutons
  SendDlgItemMessage(hDlg, IDC_ENCODE, WM_SETTEXT, 0, (LPARAM)"Encoder");
  SendDlgItemMessage(hDlg, IDC_DECODE, WM_SETTEXT, 0, (LPARAM)"Dcoder");
  SendDlgItemMessage(hDlg, IDC_STATIC_INFO, WM_SETTEXT, 0, (LPARAM)"Termin.");
  // Ractive les boutons
  EnableWindow(GetDlgItem(hDlg, IDC_ENCODE), TRUE);
  EnableWindow(GetDlgItem(hDlg, IDC_DECODE), TRUE);
  // Rinitialise la barre de progression
  SendDlgItemMessage(hDlg, IDC_PROGRESSFILE, PBM_SETPOS, 0, 0);
  // Ferme les fichiers
  Input.close(); Output.close();
}


/*-----------------------------------------------------------------------------+
| Encodage                                                                     |
+-----------------------------------------------------------------------------*/
void OnEncode(HWND hDlg)
{
  if (!Input.eof())
  {
    unsigned char in[3], out[4];
    int i, len = 0;
    for (i = 0; i < 3; i++)
    {
      in[i] = (unsigned char) Input.get();
      if (!Input.eof())
        len++;
      else
        in[i] = 0;
    }
    if (len)
    {
      out[0] = cb64[ in[0] >> 2 ];
      out[1] = cb64[ ((in[0] & 0x03) << 4) | ((in[1] & 0xf0) >> 4) ];
      out[2] = (unsigned char) (len > 1 ? cb64[ ((in[1] & 0x0f) << 2) | ((in[2] & 0xc0) >> 6) ] : '=');
      out[3] = (unsigned char) (len > 2 ? cb64[ in[2] & 0x3f ] : '=');
      for (i = 0; i < 4; i++)
        Output.put(out[i]);
    }
  }
  else
    // Aprs la conversion
    OnEndConversion(hDlg);
}


/*-----------------------------------------------------------------------------+
| Dcodage                                                                     |
+-----------------------------------------------------------------------------*/
void OnDecode(HWND hDlg)
{
  if (!Input.eof())
  {
    unsigned char in[4], out[3], v;
    int i, len = 0;
    for (len = 0, i = 0; i < 4 && !Input.eof(); i++)
    {
      v = 0;
      while (!Input.eof() && v == 0)
      {
        v = (unsigned char) Input.get();
        v = (unsigned char) ((v < 43 || v > 122) ? 0 : cd64[ v - 43 ]);
        if (v)
          v = (unsigned char) ((v == '$') ? 0 : v - 61);
      }
      if (!Input.eof())
      {
        len++;
        if (v)
          in[i] = (unsigned char) (v - 1);
      }
      else
        in[i] = 0;
    }
    if (len)
    {
      out[0] = (unsigned char) (in[0] << 2 | in[1] >> 4);
      out[1] = (unsigned char) (in[1] << 4 | in[2] >> 2);
      out[2] = (unsigned char) (((in[2] << 6) & 0xc0) | in[3]);
      for (i = 0; i < len-1; i++)
        Output.put(out[i]);
    }
  }
  else
    // Aprs la conversion
    OnEndConversion(hDlg);
}


/*-----------------------------------------------------------------------------+
| Au lancement du logiciel                                                     |
+-----------------------------------------------------------------------------*/
void OnInitDialog(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
  // Place l'icne du logiciel en haut dans la barre de titre
  SetClassLong(hDlg, GCL_HICON, (LONG)LoadIcon((HINSTANCE)GetWindowLong(hDlg, GWL_HINSTANCE), "A"));
  // Paramtres de la barre de progression
  SendDlgItemMessage(hDlg, IDC_PROGRESSFILE, PBM_SETRANGE, 0, MAKELONG(0, 100));
  // Coche la case Texte
  CheckDlgButton(hDlg, IDC_TEXT, TRUE);
  // Montre la fentre
  ShowWindow(hDlg, SW_NORMAL);
}


/*-----------------------------------------------------------------------------+
| Conversion d'un fichier                                                      |
+-----------------------------------------------------------------------------*/
void OnConversion(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
  // Fichier source
  int Length = SendDlgItemMessage(hDlg, IDC_SOURCEFILE, WM_GETTEXTLENGTH, 0, 0);
  char Buffer1[Length+1];
  SendDlgItemMessage(hDlg, IDC_SOURCEFILE, WM_GETTEXT, (WPARAM)Length+1, (LPARAM)&Buffer1);
  // Fichier destination
  Length = SendDlgItemMessage(hDlg, IDC_DESTINATIONFILE, WM_GETTEXTLENGTH, 0, 0);
  char Buffer2[Length+1];
  SendDlgItemMessage(hDlg, IDC_DESTINATIONFILE, WM_GETTEXT, (WPARAM)Length+1, (LPARAM)&Buffer2);
  // Ouvre le fichier  lire
  Input.open(Buffer1, ifstream::binary|ios::in);
  // Si l'ouverture de la source a chou, affiche un message
  if (Input.is_open())
  {
    Output.open(Buffer2, ofstream::binary|ios::out);
    // Si l'ouverture de la destination a chou, affiche une bote d'ouverture
    if (!Output.is_open())
    {
      string File = SaveFileBox("Fichier de destination", "Tous les fichiers (*.*)\0*.*\0", 1, OFN_OVERWRITEPROMPT, hDlg);
      if (File != "")
        Output.open(File.c_str(), ofstream::binary|ios::out);
    }
    // Si l'ouverture impose a chou, affiche un message
    if (Output.is_open())
    {
      // Remet le flux en tat
      Input.clear();
      // Teste si le fichier est prt pour la lecture
      if (Input.good())
      {
        // Rcupre la taille de fichier
        FileSize = (unsigned long long)GetFileSize(Input);
        // Active la barre de progression
        SetTimer(hDlg, 0, 200, NULL);
        // Encodage
        if (LOWORD(wParam) == IDC_ENCODE)
        {
          SendDlgItemMessage(hDlg, IDC_ENCODE, WM_SETTEXT, 0, (LPARAM)"Annuler");
          EnableWindow(GetDlgItem(hDlg, IDC_DECODE), FALSE);
          CallBackJob = JOB_ENCODE;
        }
        // Dcodage
        if (LOWORD(wParam) == IDC_DECODE)
        {
          SendDlgItemMessage(hDlg, IDC_DECODE, WM_SETTEXT, 0, (LPARAM)"Annuler");
          EnableWindow(GetDlgItem(hDlg, IDC_ENCODE), FALSE);
          CallBackJob = JOB_DECODE;
        }
      }
      else
      {
        // Ferme les fichiers ouverts
        Input.close(); Output.close();
        MessageBox(hDlg, "Erreur lors de la lecture du fichier.", "Erreur", MB_ICONERROR);
      }
    }
    else
    {
      Input.close();
      MessageBox(hDlg, "Veuillez choisir un fichier de destination valide.", "Erreur", MB_ICONWARNING);
    }
  }
  else
    MessageBox(hDlg, "Veuillez choisir un fichier source valide.", "Erreur", MB_ICONWARNING);
}


/*-----------------------------------------------------------------------------+
| Contrles                                                                    |
+-----------------------------------------------------------------------------*/
void OnCommand(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
  switch (LOWORD(wParam))
  {
    // Boutons checkbox
    case IDC_TEXT:
    case IDC_FILE:
    {
      int TextState = SW_SHOW;
      int FileState = SW_HIDE;
      if (IsDlgButtonChecked(hDlg, IDC_FILE))
      {
        TextState = SW_HIDE;
        FileState = SW_SHOW;
      }
      // Elments lis au texte
      ShowWindow(GetDlgItem(hDlg, IDC_STATIC_TDEC), TextState);
      ShowWindow(GetDlgItem(hDlg, IDC_STATIC_TENC), TextState);
      ShowWindow(GetDlgItem(hDlg, IDC_EDITDECODED), TextState);
      ShowWindow(GetDlgItem(hDlg, IDC_EDITENCODED), TextState);
      // Elments lis aux fichiers
      ShowWindow(GetDlgItem(hDlg, IDC_STATIC_FSRC), FileState);
      ShowWindow(GetDlgItem(hDlg, IDC_STATIC_FDST), FileState);
      ShowWindow(GetDlgItem(hDlg, IDC_SOURCEFILE), FileState);
      ShowWindow(GetDlgItem(hDlg, IDC_DESTINATIONFILE), FileState);
      ShowWindow(GetDlgItem(hDlg, IDC_FINDSOURCE), FileState);
      ShowWindow(GetDlgItem(hDlg, IDC_FINDDESTINATION), FileState);
      ShowWindow(GetDlgItem(hDlg, IDC_STATIC_INFO), FileState);
      ShowWindow(GetDlgItem(hDlg, IDC_PROGRESSFILE), FileState);
    } break;

    // Boutons parcourir
    case IDC_FINDSOURCE:
    {
      string File = OpenFileBox("Fichier source", "Tous les fichiers (*.*)\0*.*\0", 1, 0, hDlg);
      if (File != "")
        SendDlgItemMessage(hDlg, IDC_SOURCEFILE, WM_SETTEXT, 0, (LPARAM)File.c_str());
    } break;
    case IDC_FINDDESTINATION:
    {
      string File = SaveFileBox("Fichier de destination", "Tous les fichiers (*.*)\0*.*\0", 1, OFN_OVERWRITEPROMPT, hDlg);
      if (File != "")
        SendDlgItemMessage(hDlg, IDC_DESTINATIONFILE, WM_SETTEXT, 0, (LPARAM)File.c_str());
    } break;

    // Bouton Encoder
    case IDC_ENCODE:
      if (HIWORD(wParam) == BN_CLICKED)
      {
        if (IsDlgButtonChecked(hDlg, IDC_TEXT))
        {
          int Length = SendDlgItemMessage(hDlg, IDC_EDITDECODED, WM_GETTEXTLENGTH, 0, 0);
          char Buffer[Length+2];
          SendDlgItemMessage(hDlg, IDC_EDITDECODED, WM_GETTEXT, (WPARAM)Length+1, (LPARAM)&Buffer);
          SendDlgItemMessage(hDlg, IDC_EDITENCODED, WM_SETTEXT, 0, (LPARAM)Base64Encode(Buffer).c_str());
        }
        else
        {
          if (CallBackJob == JOB_NONE)
            OnConversion(hDlg, wParam, lParam);
          else
            OnEndConversion(hDlg);
        }
      }
      break;

    // Bouton Dcoder
    case IDC_DECODE:
      if (HIWORD(wParam) == BN_CLICKED)
      {
        if (IsDlgButtonChecked(hDlg, IDC_TEXT))
        {
          int Length = SendDlgItemMessage(hDlg, IDC_EDITENCODED, WM_GETTEXTLENGTH, 0, 0);
          char Buffer[Length+2];
          SendDlgItemMessage(hDlg, IDC_EDITENCODED, WM_GETTEXT, (WPARAM)Length+1, (LPARAM)&Buffer);
          SendDlgItemMessage(hDlg, IDC_EDITDECODED, WM_SETTEXT, 0, (LPARAM)Base64Decode(Buffer).c_str());
        }
        else
        {
          if (CallBackJob == JOB_NONE)
            OnConversion(hDlg, wParam, lParam);
          else
            OnEndConversion(hDlg);
        }
      }
      break;
  }
}


/*-----------------------------------------------------------------------------+
| Timer                                                                        |
+-----------------------------------------------------------------------------*/
void OnTimer(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
  // Calcule le pourcentage de progression
  char Status = 0;
  if (FileSize != 0)
    Status = (char)((Input.tellg()*100)/FileSize);
  // Met  jour la barre de progression
  SendDlgItemMessage(hDlg, IDC_PROGRESSFILE, PBM_SETPOS, Status, 0);
  // Met  jour la barre d'informations
  char Buffer[128];
  sprintf(Buffer, "Convertis : %d octets", (unsigned long long)Input.tellg());
  SendDlgItemMessage(hDlg, IDC_STATIC_INFO, WM_SETTEXT, 0, (LPARAM)Buffer);
}


/*-----------------------------------------------------------------------------+
| A la fermeture                                                               |
+-----------------------------------------------------------------------------*/
void OnClose(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
  // Si un job tait en cours
  if (CallBackJob != JOB_NONE)
    OnEndConversion(hDlg);
  // Ferme la bote de dialogue
  EndDialog(hDlg, 0);
  // Poste un message de fin
  PostQuitMessage(0);
}


/*-----------------------------------------------------------------------------+
| Callback de la fentre principale                                            |
+-----------------------------------------------------------------------------*/
BOOL APIENTRY DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  /* Slection des messages */
  switch (uMsg)
  {
    // Au lancement du logiciel
    case WM_INITDIALOG:
      OnInitDialog(hDlg, wParam, lParam);
      break;

    // L'utilisateur a cliqu sur un contrle
    case WM_COMMAND:
      OnCommand(hDlg, wParam, lParam);
      break;

    // Evnement de timer
    case WM_TIMER:
      OnTimer(hDlg, wParam, lParam);
      break;

    // L'utilisateur ferme le fentre
    case WM_CLOSE:
      OnClose(hDlg, wParam, lParam);
      break;

    // Evnement non pris en charge
    default:
      // On laisse Windows s'occuper de l'vnement inconnu
      return FALSE;
  }
  // On dit  Windows qu'on s'est occups de l'vnement
  return TRUE;
}


/*-----------------------------------------------------------------------------+
| WinMain                                                                      |
+-----------------------------------------------------------------------------*/
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{

  /* Initialisation des contrles communs */
  INITCOMMONCONTROLSEX lpInitCtrls;
  lpInitCtrls.dwSize = sizeof(lpInitCtrls);
  lpInitCtrls.dwICC = ICC_BAR_CLASSES;
  if (InitCommonControlsEx(&lpInitCtrls) == FALSE)
  {
    MessageBox(0, "Impossible d'initialiser les contrles communs.", "Erreur", MB_ICONERROR);
    return 0;
  }

  /* Ouverture de la fentre */
  MSG msg;
	HWND hWnd = CreateDialog(hInstance, MAKEINTRESOURCE(IDD_DIALOG), NULL, DialogProc);
	for (;;)
  {
    while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
    {
      if (msg.message == WM_QUIT)
        return 0;
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
    switch (CallBackJob)
    {
      case JOB_ENCODE:
        OnEncode(hWnd);
        break;
      case JOB_DECODE:
        OnDecode(hWnd);
        break;
      default:
        WaitMessage(); // Will not return until a message is posted.
        break;
    }
  }
}

/* Fin de main.cpp */
